Class: CopyAmi

Inherits:
Ec2Script show all
Defined in:
lib/scripts/ec2/copy_ami.rb

Overview

Copy a given snapshot to another region

  • start up instance in source-region, create a snapshot from the mounted EBS
  • then create volume from snapshot, attach volume, and mount it
  • start up instance in destination-region, create empty volume of same size, attache volume, and mount it
  • copy the destination key to the source instance
  • perform an rsynch sync -PHAXaz --rsh "ssh -i /home/$src_user/.ssh/id_$dst_keypair" --rsync-path "sudo rsync" $src_dir/ $dst_user@$dst_public_fqdn:$dst_dir/
  • create a snapshot of the volume
  • register the snapshot as AMI
  • clean-up everything

Defined Under Namespace

Classes: AmiRegisteredState, AmiSnapshotCreatedState, CopyAmiState, DataCopiedState, InitialState, KeyInPlaceState, SourceCleanedUpState, SourceInstanceLaunchedState, SourceVolumeReadyState, TargetInstanceLaunchedState, TargetSnapshotCreatedState, TargetVolumeReadyState

Instance Method Summary collapse

Methods inherited from Ec2Script

#get_execution_result, #post_message, #register_progress_message_listener, #register_state_change_listener, #start_script

Constructor Details

#initialize(input_params) ⇒ CopyAmi

context information needed

  • the EC2 credentials (see #Ec2Script)
  • ami_id => the ID of the AMI to be copied in another region
  • target_ec2_handler => The EC2 handler connected to the region where the snapshot is being copied to
  • source_ssh_username => The username for ssh for source-instance (default = root)
  • source_key_name => Key name of the instance that manages the snaphot-volume in the source region
  • source_ssh_key_data => Key information for the security group that starts the AMI [if not set, use ssh_key_files]
  • source_ssh_key_files => Key information for the security group that starts the AMI
  • target_ssh_username => The username for ssh for target-instance (default = root)
  • target_key_name => Key name of the instance that manages the snaphot-volume in the target region
  • target_ssh_key_data => Key information for the security group that starts the AMI [if not set, use ssh_key_files]
  • target_ssh_key_files => Key information for the security group that starts the AMI
  • target_ami_id => ID of the AMI to start in the target region
  • name => name of new AMI to be created
  • description => description of new AMI to be created


36
37
38
# File 'lib/scripts/ec2/copy_ami.rb', line 36

def initialize(input_params)
  super(input_params)
end

Instance Method Details

#check_input_parameters ⇒ Object



40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
# File 'lib/scripts/ec2/copy_ami.rb', line 40

def check_input_parameters()
  if @input_params[:ami_id] == nil && !(@input_params[:ami_id] =~ /^ami-.*$/)
    raise Exception.new("Invalid AMI ID specified: #{@input_params[:ami_id]}")
  end
  ec2_helper = Ec2Helper.new(@input_params[:ec2_api_handler])
  if ec2_helper.ami_prop(@input_params[:ami_id], 'rootDeviceType') != "ebs"
    raise Exception.new("must be an EBS type image")
  end
  local_ec2_helper = ec2_helper
  if !local_ec2_helper.check_open_port('default', 22)
    raise Exception.new("Port 22 must be opened for security group 'default' to connect via SSH in source-region")
  end
  remote_ec2_helper = Ec2Helper.new(@input_params[:target_ec2_handler])
  if !remote_ec2_helper.check_open_port('default', 22)
    raise Exception.new("Port 22 must be opened for security group 'default' to connect via SSH in target-region")
  end
  if @input_params[:root_device_name] == nil
    @input_params[:root_device_name] = "/dev/sda1"
  end
  if @input_params[:temp_device_name] == nil
    @input_params[:temp_device_name] = "/dev/sdj"
  end
  if @input_params[:source_ssh_username] == nil
    @input_params[:source_ssh_username] = "root"
  end
  if @input_params[:target_ssh_username] == nil
    @input_params[:target_ssh_username] = "root"
  end
end

#load_initial_state ⇒ Object

Load the initial state for the script. Abstract method to be implemented by extending classes.



72
73
74
# File 'lib/scripts/ec2/copy_ami.rb', line 72

def load_initial_state()
  CopyAmiState.load_state(@input_params)
end