Class: RemoteCommandHandler
- Inherits:
-
Object
- Object
- RemoteCommandHandler
- Defined in:
- lib/help/remote_command_handler.rb
Overview
Provides methods to be executed via ssh to remote instances.
Direct Known Subclasses
Instance Attribute Summary collapse
-
#logger ⇒ Object
Returns the value of attribute logger.
-
#ssh_session ⇒ Object
Returns the value of attribute ssh_session.
-
#use_sudo ⇒ Object
Returns the value of attribute use_sudo.
Instance Method Summary collapse
-
#connect(ip, user, key_data, timeout = 30) ⇒ Object
Connect to the machine as root using keydata from a keyfile.
-
#connect_with_keyfile(ip, user_name, keyfile, timeout = 30) ⇒ Object
Connect to the machine as root using a keyfile.
- #create_filesystem(fs_type, volume) ⇒ Object
-
#disable_sudoers_requiretty ⇒ Object
Disable ‘Defaults requiretty’ option in sudoers file.
-
#disconnect ⇒ Object
Disconnect the current handler.
-
#drive_mounted?(path) ⇒ Boolean
Checks if the drive on path is mounted.
-
#drive_mounted_as?(device, path) ⇒ Boolean
Checks if the drive on path is mounted with the specific device.
- #echo(data, file) ⇒ Object
-
#enable_sudoers_requiretty ⇒ Object
Enable ‘Defaults requiretty’ option in sudoers file.
-
#file_exists?(path) ⇒ Boolean
Check if the path/file specified exists.
-
#get_device_label(device) ⇒ Object
Get device label.
-
#get_device_label_ext(device, fs_type) ⇒ Object
Get device label.
-
#get_output(exec_string, push_data = nil, stdout = [], stderr = []) ⇒ Object
Executes the specified #exec_string on a remote session specified as #ssh_session.
-
#get_partition_device(part) ⇒ Object
Get partition label.
-
#get_partition_fs_type(part) ⇒ Object
Get filesystem type.
-
#get_root_device ⇒ Object
Get root partition label.
-
#get_root_fs_type ⇒ Object
Get filesystem type.
-
#initialize ⇒ RemoteCommandHandler
constructor
A new instance of RemoteCommandHandler.
-
#install(software_package) ⇒ Object
Installs the software package specified.
-
#is_port_open?(ip, port) ⇒ Boolean
Checks for a given IP/port if there’s a response on that port.
-
#local_rcopy(source_path, dest_path, exclude_path = nil) ⇒ Object
Copy directory using basic cp exclude_path: a space separated list of directory.
-
#local_rsync(source_path, dest_path, exclude_path = nil) ⇒ Object
Copy directory using options -avHx.
- #mkdir(path) ⇒ Object
- #mount(device, path) ⇒ Object
-
#remote_execute(exec_string, push_data = nil, raise_exception = false) ⇒ Object
Executes the specified #exec_string on a remote session specified.
- #remote_rsync(keyfile, source_path, dest_ip, dest_user, dest_path) ⇒ Object
-
#remote_rsync_old(keyfile, source_path, dest_ip, dest_path) ⇒ Object
Rsync directory via an ssh-tunnel.
-
#retrieve_os ⇒ Object
Returns the result of uname -a (Linux).
-
#scp(keyfile, source_path, dest_ip, dest_user, dest_path) ⇒ Object
Copy directory via an ssh-tunnel.
-
#set_device_label(device, label) ⇒ Object
Set device label.
-
#set_device_label_ext(device, label, fs_type) ⇒ Object
Set device label.
-
#stdout_contains?(exec_string, search_string = "", push_data = nil) ⇒ Boolean
Executes the specified #exec_string on a remote session specified as #ssh_session and logs the command-output into the specified #logger.
-
#tools_installed?(software_package) ⇒ Boolean
Checks if the software package specified is installed.
-
#umount(path) ⇒ Object
Unmount the specified path.
- #upload(ip, user, key_data, local_file, destination_file, timeout = 60) ⇒ Object
-
#zip(source_path, destination_file) ⇒ Object
Zip the complete contents of the source path into the destination file.
Constructor Details
#initialize ⇒ RemoteCommandHandler
Returns a new instance of RemoteCommandHandler.
9 10 11 12 |
# File 'lib/help/remote_command_handler.rb', line 9 def initialize() @logger = Logger.new(STDOUT) @use_sudo = false end |
Instance Attribute Details
#logger ⇒ Object
Returns the value of attribute logger.
8 9 10 |
# File 'lib/help/remote_command_handler.rb', line 8 def logger @logger end |
#ssh_session ⇒ Object
Returns the value of attribute ssh_session.
8 9 10 |
# File 'lib/help/remote_command_handler.rb', line 8 def ssh_session @ssh_session end |
#use_sudo ⇒ Object
Returns the value of attribute use_sudo.
8 9 10 |
# File 'lib/help/remote_command_handler.rb', line 8 def use_sudo @use_sudo end |
Instance Method Details
#connect(ip, user, key_data, timeout = 30) ⇒ Object
Connect to the machine as root using keydata from a keyfile. Params:
-
ip: ip address of the machine to connect to
-
user: user name
-
key_data: key_data to be used for authentication
46 47 48 49 50 |
# File 'lib/help/remote_command_handler.rb', line 46 def connect(ip, user, key_data, timeout = 30) @use_sudo = false @ssh_session = Net::SSH.start(ip, user, :key_data => [key_data], :timeout => timeout, :verbose => :warn) @use_sudo = true unless user.strip == 'root' end |
#connect_with_keyfile(ip, user_name, keyfile, timeout = 30) ⇒ Object
Connect to the machine as root using a keyfile. Params:
-
ip: ip address of the machine to connect to
-
keyfile: path of the keyfile to be used for authentication
35 36 37 38 39 |
# File 'lib/help/remote_command_handler.rb', line 35 def connect_with_keyfile(ip, user_name, keyfile, timeout = 30) @use_sudo = false @ssh_session = Net::SSH.start(ip, user_name, :keys => [keyfile], :timeout => timeout, :verbose => :warn) @use_sudo = true unless user_name.strip == 'root' end |
#create_filesystem(fs_type, volume) ⇒ Object
138 139 140 141 142 |
# File 'lib/help/remote_command_handler.rb', line 138 def create_filesystem(fs_type, volume) e = "mkfs -t #{fs_type} #{volume}" #remote_execute(e, "y") #TODO: quiet mode? remote_execute(e, nil, false) end |
#disable_sudoers_requiretty ⇒ Object
Disable ‘Defaults requiretty’ option in sudoers file
213 214 215 216 217 218 219 220 |
# File 'lib/help/remote_command_handler.rb', line 213 def disable_sudoers_requiretty() e = "sed -r -e \'s/^(Defaults[[:blank:]]+requiretty)$/# \\1/\' -i /etc/sudoers" @logger.debug "going to execute '#{e}'" status = remote_exec_helper(e, nil, nil, true) if status != true raise Exception.new("disabling 'requiretty' from sudoers failed with status: #{status}") end end |
#disconnect ⇒ Object
Disconnect the current handler
53 54 55 |
# File 'lib/help/remote_command_handler.rb', line 53 def disconnect @ssh_session.close end |
#drive_mounted?(path) ⇒ Boolean
Checks if the drive on path is mounted
155 156 157 158 159 160 161 162 163 164 |
# File 'lib/help/remote_command_handler.rb', line 155 def drive_mounted?(path) #check if drive mounted drive_found = stdout_contains?("mount", "on #{path} type") if drive_found return file_exists?(path) else @logger.debug "not mounted (since #{path} non-existing)" false end end |
#drive_mounted_as?(device, path) ⇒ Boolean
Checks if the drive on path is mounted with the specific device
167 168 169 170 |
# File 'lib/help/remote_command_handler.rb', line 167 def drive_mounted_as?(device, path) #check if drive mounted stdout_contains?("mount", "#{device} on #{path} type") end |
#echo(data, file) ⇒ Object
262 263 264 265 266 267 268 269 |
# File 'lib/help/remote_command_handler.rb', line 262 def echo(data, file) exec = "echo #{data} > #{file}" @logger.debug "going to execute #{exec}" remote_execute(exec, nil, true) if !file_exists?(file) raise Exception.new("file #{file} could not be created") end end |
#enable_sudoers_requiretty ⇒ Object
Enable ‘Defaults requiretty’ option in sudoers file
223 224 225 226 227 228 229 230 |
# File 'lib/help/remote_command_handler.rb', line 223 def enable_sudoers_requiretty() e = "sed -r -e \'s/^#[[:blank:]]*(Defaults[[:blank:]]+requiretty)$/\\1/\' -i /etc/sudoers" @logger.debug "going to execute '#{e}'" status = remote_exec_helper(e, nil, nil, true) if status != true raise Exception.new("enabling 'requiretty' from sudoers failed with status: #{status}") end end |
#file_exists?(path) ⇒ Boolean
Check if the path/file specified exists
58 59 60 |
# File 'lib/help/remote_command_handler.rb', line 58 def file_exists?(path) remote_execute("ls #{path}") end |
#get_device_label(device) ⇒ Object
Get device label
78 79 80 |
# File 'lib/help/remote_command_handler.rb', line 78 def get_device_label(device) get_output("e2label #{device}").strip end |
#get_device_label_ext(device, fs_type) ⇒ Object
Get device label
83 84 85 86 87 88 89 90 |
# File 'lib/help/remote_command_handler.rb', line 83 def get_device_label_ext(device, fs_type) if fs_type.eql?("xfs") cmd = "xfs_admin -l #{device} | sed -r -e 's/^label[[:blank:]]*=[[:blank:]]*\"(.*)\"$/\\1/'" else cmd = "e2label #{device}" end get_output(cmd).strip end |
#get_output(exec_string, push_data = nil, stdout = [], stderr = []) ⇒ Object
Executes the specified #exec_string on a remote session specified as #ssh_session. When #push_data is specified, the data will be used as input for the command and thus allows to respond in advance to commands that ask the user something. It returns stdout. When #stdout or #stderr is specified as arrays, the respective output is also written into those arrays.
309 310 311 312 313 314 315 |
# File 'lib/help/remote_command_handler.rb', line 309 def get_output(exec_string, push_data = nil, stdout = [], stderr = []) exec_string = "echo #{push_data} >tmp.txt; #{exec_string} <tmp.txt; rm -f tmp.txt" unless push_data == nil stdout = [] stderr = [] remote_exec_helper(exec_string, stdout, stderr, true) stdout.join() end |
#get_partition_device(part) ⇒ Object
Get partition label
73 74 75 |
# File 'lib/help/remote_command_handler.rb', line 73 def get_partition_device(part) get_output("cat /etc/mtab | grep -E '[[:blank:]]+" + "#{part}" + "[[:blank:]]+' | cut -d ' ' -f 1").strip end |
#get_partition_fs_type(part) ⇒ Object
Get filesystem type
113 114 115 |
# File 'lib/help/remote_command_handler.rb', line 113 def get_partition_fs_type(part) get_output("cat /etc/mtab | grep -E '[[:blank:]]+" + "#{part}" + "[[:blank:]]+' | cut -d ' ' -f 3").strip end |
#get_root_device ⇒ Object
Get root partition label
68 69 70 |
# File 'lib/help/remote_command_handler.rb', line 68 def get_root_device() get_output("cat /etc/mtab | grep -E '[[:blank:]]+\/[[:blank:]]+' | cut -d ' ' -f 1").strip end |
#get_root_fs_type ⇒ Object
Get filesystem type
108 109 110 |
# File 'lib/help/remote_command_handler.rb', line 108 def get_root_fs_type() get_output("cat /etc/mtab | grep -E '[[:blank:]]+\/[[:blank:]]+' | cut -d ' ' -f 3").strip end |
#install(software_package) ⇒ Object
Installs the software package specified.
118 119 120 121 122 123 124 125 126 127 |
# File 'lib/help/remote_command_handler.rb', line 118 def install(software_package) e = "yum -yq install #{software_package}" yum = remote_execute(e) if !yum @logger.info("yum installation failed; try apt-get") e = "apt-get -yq install #{software_package}" apt = remote_execute(e) @logger.info("apt=get installation? #{apt}") end end |
#is_port_open?(ip, port) ⇒ Boolean
Checks for a given IP/port if there’s a response on that port.
15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 |
# File 'lib/help/remote_command_handler.rb', line 15 def is_port_open?(ip, port) begin Timeout::timeout(5) do begin s = TCPSocket.new(ip, port) s.close return true rescue Errno::ECONNREFUSED, Errno::EHOSTUNREACH return false end end rescue Timeout::Error return false end end |
#local_rcopy(source_path, dest_path, exclude_path = nil) ⇒ Object
Copy directory using basic cp exclude_path: a space separated list of directory
181 182 183 184 185 186 187 188 189 190 191 192 |
# File 'lib/help/remote_command_handler.rb', line 181 def local_rcopy(source_path, dest_path, exclude_path = nil) e = "" if exclude_path.nil? || exclude_path.empty? e = "cp -Rpv #{source_path} #{dest_path}" else # only one level of exclusion exclusion_regexp = exclude_path.gsub(' ', '|') e = "for dir in $(ls -d #{source_path}* | grep -E -v '#{exclusion_regexp}'); do cp -Rpv $dir #{dest_path}; done;" end @logger.debug "going to execute #{e}" remote_exec_helper(e, nil, nil, false) end |
#local_rsync(source_path, dest_path, exclude_path = nil) ⇒ Object
Copy directory using options -avHx
195 196 197 198 199 200 201 202 203 |
# File 'lib/help/remote_command_handler.rb', line 195 def local_rsync(source_path, dest_path, exclude_path = nil) exclude = "" if exclude_path != nil exclude = "--exclude #{exclude_path}" end e = "rsync -avHx #{exclude} #{source_path} #{dest_path}" @logger.debug "going to execute #{e}" remote_exec_helper(e, nil, nil, true) #TODO: handle output in stderr? end |
#mkdir(path) ⇒ Object
144 145 146 147 |
# File 'lib/help/remote_command_handler.rb', line 144 def mkdir(path) e = "mkdir #{path}" remote_execute(e, nil, true) end |
#mount(device, path) ⇒ Object
149 150 151 152 |
# File 'lib/help/remote_command_handler.rb', line 149 def mount(device, path) e = "mount #{device} #{path}" remote_execute(e, nil, true) end |
#remote_execute(exec_string, push_data = nil, raise_exception = false) ⇒ Object
Executes the specified #exec_string on a remote session specified. When #push_data is specified, the data will be used as input for the command and thus allow to respond in advance to commands that ask the user something. The method will return true if nothing was written into stderr, otherwise false. When #raise_exception is set, an exception will be raised instead of returning false.
278 279 280 281 282 283 284 285 286 287 |
# File 'lib/help/remote_command_handler.rb', line 278 def remote_execute(exec_string, push_data = nil, raise_exception = false) exec_string = "echo #{push_data} >tmp.txt; #{exec_string} <tmp.txt; rm -f tmp.txt" unless push_data == nil stdout = [] stderr = [] result = remote_exec_helper(exec_string, stdout, stderr) em = "RemoteCommandHandler: #{exec_string} lead to stderr message: #{stderr.join().strip}" @logger.info(em) unless stderr.size == 0 raise Exception.new(em) unless result == true || raise_exception == false result end |
#remote_rsync(keyfile, source_path, dest_ip, dest_user, dest_path) ⇒ Object
232 233 234 235 236 237 238 239 240 |
# File 'lib/help/remote_command_handler.rb', line 232 def remote_rsync(keyfile, source_path, dest_ip, dest_user, dest_path) e = "rsync -rlpgoDzq --rsh 'ssh -o stricthostkeychecking=no -i #{keyfile}' --rsync-path='sudo rsync'"+ " #{source_path} #{dest_user}@#{dest_ip}:#{dest_path}" @logger.debug "going to execute #{e}" status = remote_exec_helper(e, nil, nil, true) #TODO: handle output in stderr? if status != true raise Exception.new("rsync bewteen source and target servers failed with status: #{status}") end end |
#remote_rsync_old(keyfile, source_path, dest_ip, dest_path) ⇒ Object
Rsync directory via an ssh-tunnel.
206 207 208 209 210 |
# File 'lib/help/remote_command_handler.rb', line 206 def remote_rsync_old(keyfile, source_path, dest_ip, dest_path) e = "rsync -rlpgoDzq -e "+'"'+"ssh -o stricthostkeychecking=no -i #{keyfile}"+'"'+" #{source_path} root@#{dest_ip}:#{dest_path}" @logger.debug "going to execute #{e}" remote_exec_helper(e, nil, nil, false) #TODO: handle output in stderr? end |
#retrieve_os ⇒ Object
Returns the result of uname -a (Linux)
63 64 65 |
# File 'lib/help/remote_command_handler.rb', line 63 def retrieve_os() get_output("uname -r").strip end |
#scp(keyfile, source_path, dest_ip, dest_user, dest_path) ⇒ Object
Copy directory via an ssh-tunnel.
243 244 245 246 247 |
# File 'lib/help/remote_command_handler.rb', line 243 def scp(keyfile, source_path, dest_ip, dest_user, dest_path) e = "scp -Cpqr -o stricthostkeychecking=no -i #{keyfile} #{source_path} #{dest_user}@#{dest_ip}:#{dest_path}" @logger.debug "going to execute #{e}" remote_exec_helper(e, nil, nil, false) #TODO: handle output in stderr? end |
#set_device_label(device, label) ⇒ Object
Set device label
93 94 95 |
# File 'lib/help/remote_command_handler.rb', line 93 def set_device_label(device, label) remote_execute("e2label #{device} #{label}", nil, false) end |
#set_device_label_ext(device, label, fs_type) ⇒ Object
Set device label
98 99 100 101 102 103 104 105 |
# File 'lib/help/remote_command_handler.rb', line 98 def set_device_label_ext(device, label, fs_type) if fs_type.eql?("xfs") cmd = "xfs_admin -L #{label} #{device}" else cmd = "e2label #{device} #{label}" end remote_execute(cmd, nil, false) end |
#stdout_contains?(exec_string, search_string = "", push_data = nil) ⇒ Boolean
Executes the specified #exec_string on a remote session specified as #ssh_session and logs the command-output into the specified #logger. When #push_data is specified, the data will be used as input for the command and thus allows to respond in advance to commands that ask the user something. If the output in stdout contains the specified #search_string, the method returns true otherwise false. Output to stderr will be logged.
295 296 297 298 299 300 301 302 |
# File 'lib/help/remote_command_handler.rb', line 295 def stdout_contains?(exec_string, search_string = "", push_data = nil) exec_string = "echo #{push_data} >tmp.txt; #{exec_string} <tmp.txt; rm -f tmp.txt" unless push_data == nil stdout = [] stderr = [] remote_exec_helper(exec_string, stdout, stderr) @logger.info("RemoteCommandHandler: #{exec_string} lead to stderr message: #{stderr.join().strip}") unless stderr.size == 0 stdout.join().include?(search_string) end |
#tools_installed?(software_package) ⇒ Boolean
Checks if the software package specified is installed.
130 131 132 133 134 135 136 |
# File 'lib/help/remote_command_handler.rb', line 130 def tools_installed?(software_package) exec_string = "which #{software_package}" stdout = [] stderr = [] result = remote_exec_helper(exec_string, stdout, stderr) return result == true && stdout.size > 0 end |
#umount(path) ⇒ Object
Unmount the specified path.
173 174 175 176 177 |
# File 'lib/help/remote_command_handler.rb', line 173 def umount(path) exec_string = "umount #{path}" remote_execute(exec_string) !drive_mounted?(path) end |
#upload(ip, user, key_data, local_file, destination_file, timeout = 60) ⇒ Object
317 318 319 320 321 322 323 |
# File 'lib/help/remote_command_handler.rb', line 317 def upload(ip, user, key_data, local_file, destination_file, timeout = 60) Timeout::timeout(timeout) { Net::SCP.start(ip, user, {:key_data => [key_data], :timeout => timeout}) do |scp| scp.upload!(local_file, destination_file) end } end |
#zip(source_path, destination_file) ⇒ Object
Zip the complete contents of the source path into the destination file. Returns the an array with stderr output messages.
251 252 253 254 255 256 257 258 259 260 |
# File 'lib/help/remote_command_handler.rb', line 251 def zip(source_path, destination_file) begin exec = "cd #{source_path}; zip -ryq #{destination_file} *" stderr = [] get_output(exec, nil, nil, stderr) return stderr rescue Exception => e raise Exception.new("zip failed due to #{e.}") end end |